@@ -314,8 +314,12 @@ class BaseModelAdmin(metaclass=forms.MediaDefiningClass):
                 kwargs["queryset"] = queryset
 
         form_field = db_field.formfield(**kwargs)
-        if isinstance(form_field.widget, SelectMultiple) and not isinstance(
-            form_field.widget, (CheckboxSelectMultiple, AutocompleteSelectMultiple)
+        if (
+            isinstance(form_field.widget, SelectMultiple)
+            and form_field.widget.allow_multiple_selected
+            and not isinstance(
+                form_field.widget, (CheckboxSelectMultiple, AutocompleteSelectMultiple)
+            )
         ):
             msg = _(
                 "Hold down “Control”, or “Command” on a Mac, to select more than one."
@@ -490,10 +494,8 @@ class BaseModelAdmin(metaclass=forms.MediaDefiningClass):
         Return True if the model associated with this admin should be
         allowed to be referenced by the specified field.
         """
-        opts = self.model._meta
-
         try:
-            field = opts.get_field(to_field)
+            field = self.opts.get_field(to_field)
         except FieldDoesNotExist:
             return False
 
@@ -504,7 +506,7 @@ class BaseModelAdmin(metaclass=forms.MediaDefiningClass):
 
         # Allow reverse relationships to models defining m2m fields if they
         # target the specified field.
-        for many_to_many in opts.many_to_many:
+        for many_to_many in self.opts.many_to_many:
             if many_to_many.m2m_target_field_name() == to_field:
                 return True
 
@@ -518,7 +520,7 @@ class BaseModelAdmin(metaclass=forms.MediaDefiningClass):
 
         related_objects = (
             f
-            for f in opts.get_fields(include_hidden=True)
+            for f in self.opts.get_fields(include_hidden=True)
             if (f.auto_created and not f.concrete)
         )
         for related_object in related_objects:
@@ -559,7 +561,7 @@ class BaseModelAdmin(metaclass=forms.MediaDefiningClass):
 
     def has_delete_permission(self, request, obj=None):
         """
-        Return True if the given request has permission to change the given
+        Return True if the given request has permission to delete the given
         Django model instance, the default implementation doesn't examine the
         `obj` parameter.
 
@@ -653,7 +655,7 @@ class ModelAdmin(BaseModelAdmin):
         super().__init__()
 
     def __str__(self):
-        return "%s.%s" % (self.model._meta.app_label, self.__class__.__name__)
+        return "%s.%s" % (self.opts.app_label, self.__class__.__name__)
 
     def __repr__(self):
         return (
@@ -688,7 +690,7 @@ class ModelAdmin(BaseModelAdmin):
             wrapper.model_admin = self
             return update_wrapper(wrapper, view)
 
-        info = self.model._meta.app_label, self.model._meta.model_name
+        info = self.opts.app_label, self.opts.model_name
 
         return [
             path("", wrap(self.changelist_view), name="%s_%s_changelist" % info),
@@ -1145,12 +1147,12 @@ class ModelAdmin(BaseModelAdmin):
             for bit in smart_split(search_term):
                 if bit.startswith(('"', "'")) and bit[0] == bit[-1]:
                     bit = unescape_string_literal(bit)
-                or_queries = models.Q(
-                    *((orm_lookup, bit) for orm_lookup in orm_lookups),
-                    _connector=models.Q.OR,
+                or_queries = models.Q.create(
+                    [(orm_lookup, bit) for orm_lookup in orm_lookups],
+                    connector=models.Q.OR,
                 )
                 term_queries.append(or_queries)
-            queryset = queryset.filter(models.Q(*term_queries))
+            queryset = queryset.filter(models.Q.create(term_queries))
             may_have_duplicates |= any(
                 lookup_spawns_duplicates(self.opts, search_spec)
                 for search_spec in orm_lookups
@@ -1163,11 +1165,10 @@ class ModelAdmin(BaseModelAdmin):
         """
         match = request.resolver_match
         if self.preserve_filters and match:
-            opts = self.model._meta
             current_url = "%s:%s" % (match.app_name, match.url_name)
             changelist_url = "admin:%s_%s_changelist" % (
-                opts.app_label,
-                opts.model_name,
+                self.opts.app_label,
+                self.opts.model_name,
             )
             if current_url == changelist_url:
                 preserved_filters = request.GET.urlencode()
@@ -1256,11 +1257,10 @@ class ModelAdmin(BaseModelAdmin):
     def render_change_form(
         self, request, context, add=False, change=False, form_url="", obj=None
     ):
-        opts = self.model._meta
-        app_label = opts.app_label
+        app_label = self.opts.app_label
         preserved_filters = self.get_preserved_filters(request)
         form_url = add_preserved_filters(
-            {"preserved_filters": preserved_filters, "opts": opts}, form_url
+            {"preserved_filters": preserved_filters, "opts": self.opts}, form_url
         )
         view_on_site_url = self.get_view_on_site_url(obj)
         has_editable_inline_admin_formsets = False
@@ -1291,7 +1291,7 @@ class ModelAdmin(BaseModelAdmin):
                 "has_absolute_url": view_on_site_url is not None,
                 "absolute_url": view_on_site_url,
                 "form_url": form_url,
-                "opts": opts,
+                "opts": self.opts,
                 "content_type_id": get_content_type_for_model(self.model).pk,
                 "save_as": self.save_as,
                 "save_on_top": self.save_on_top,
@@ -1311,7 +1311,7 @@ class ModelAdmin(BaseModelAdmin):
             request,
             form_template
             or [
-                "admin/%s/%s/change_form.html" % (app_label, opts.model_name),
+                "admin/%s/%s/change_form.html" % (app_label, self.opts.model_name),
                 "admin/%s/change_form.html" % app_label,
                 "admin/change_form.html",
             ],
@@ -1441,7 +1441,7 @@ class ModelAdmin(BaseModelAdmin):
                 },
             )
 
-        opts = self.model._meta
+        opts = self.opts
         preserved_filters = self.get_preserved_filters(request)
 
         msg_dict = {
@@ -1508,15 +1508,14 @@ class ModelAdmin(BaseModelAdmin):
             return self.response_post_save_change(request, obj)
 
     def _response_post_save(self, request, obj):
-        opts = self.model._meta
         if self.has_view_or_change_permission(request):
             post_url = reverse(
-                "admin:%s_%s_changelist" % (opts.app_label, opts.model_name),
+                "admin:%s_%s_changelist" % (self.opts.app_label, self.opts.model_name),
                 current_app=self.admin_site.name,
             )
             preserved_filters = self.get_preserved_filters(request)
             post_url = add_preserved_filters(
-                {"preserved_filters": preserved_filters, "opts": opts}, post_url
+                {"preserved_filters": preserved_filters, "opts": self.opts}, post_url
             )
         else:
             post_url = reverse("admin:index", current_app=self.admin_site.name)
@@ -1609,8 +1608,6 @@ class ModelAdmin(BaseModelAdmin):
         """
         Determine the HttpResponse for the delete_view stage.
         """
-        opts = self.model._meta
-
         if IS_POPUP_VAR in request.POST:
             popup_response_data = json.dumps(
                 {
@@ -1623,8 +1620,8 @@ class ModelAdmin(BaseModelAdmin):
                 self.popup_response_template
                 or [
                     "admin/%s/%s/popup_response.html"
-                    % (opts.app_label, opts.model_name),
-                    "admin/%s/popup_response.html" % opts.app_label,
+                    % (self.opts.app_label, self.opts.model_name),
+                    "admin/%s/popup_response.html" % self.opts.app_label,
                     "admin/popup_response.html",
                 ],
                 {
@@ -1636,7 +1633,7 @@ class ModelAdmin(BaseModelAdmin):
             request,
             _("The %(name)s “%(obj)s” was deleted successfully.")
             % {
-                "name": opts.verbose_name,
+                "name": self.opts.verbose_name,
                 "obj": obj_display,
             },
             messages.SUCCESS,
@@ -1644,20 +1641,19 @@ class ModelAdmin(BaseModelAdmin):
 
         if self.has_change_permission(request, None):
             post_url = reverse(
-                "admin:%s_%s_changelist" % (opts.app_label, opts.model_name),
+                "admin:%s_%s_changelist" % (self.opts.app_label, self.opts.model_name),
                 current_app=self.admin_site.name,
             )
             preserved_filters = self.get_preserved_filters(request)
             post_url = add_preserved_filters(
-                {"preserved_filters": preserved_filters, "opts": opts}, post_url
+                {"preserved_filters": preserved_filters, "opts": self.opts}, post_url
             )
         else:
             post_url = reverse("admin:index", current_app=self.admin_site.name)
         return HttpResponseRedirect(post_url)
 
     def render_delete_form(self, request, context):
-        opts = self.model._meta
-        app_label = opts.app_label
+        app_label = self.opts.app_label
 
         request.current_app = self.admin_site.name
         context.update(
@@ -1671,7 +1667,7 @@ class ModelAdmin(BaseModelAdmin):
             self.delete_confirmation_template
             or [
                 "admin/{}/{}/delete_confirmation.html".format(
-                    app_label, opts.model_name
+                    app_label, self.opts.model_name
                 ),
                 "admin/{}/delete_confirmation.html".format(app_label),
                 "admin/delete_confirmation.html",
@@ -1695,7 +1691,7 @@ class ModelAdmin(BaseModelAdmin):
                 has_change_permission = inline.has_change_permission(request, obj)
                 has_delete_permission = inline.has_delete_permission(request, obj)
             else:
-                # Disable all edit-permissions, and overide formset settings.
+                # Disable all edit-permissions, and override formset settings.
                 has_add_permission = (
                     has_change_permission
                 ) = has_delete_permission = False
@@ -1724,7 +1720,7 @@ class ModelAdmin(BaseModelAdmin):
         initial = dict(request.GET.items())
         for k in initial:
             try:
-                f = self.model._meta.get_field(k)
+                f = self.opts.get_field(k)
             except FieldDoesNotExist:
                 continue
             # We have to special-case M2Ms as a list of comma-separated PKs.
@@ -1757,9 +1753,6 @@ class ModelAdmin(BaseModelAdmin):
                 "The field %s cannot be referenced." % to_field
             )
 
-        model = self.model
-        opts = model._meta
-
         if request.method == "POST" and "_saveasnew" in request.POST:
             object_id = None
 
@@ -1781,7 +1774,9 @@ class ModelAdmin(BaseModelAdmin):
                     raise PermissionDenied
 
             if obj is None:
-                return self._get_obj_does_not_exist_redirect(request, opts, object_id)
+                return self._get_obj_does_not_exist_redirect(
+                    request, self.opts, object_id
+                )
 
         fieldsets = self.get_fieldsets(request, obj)
         ModelForm = self.get_form(
@@ -1830,7 +1825,7 @@ class ModelAdmin(BaseModelAdmin):
             readonly_fields = flatten_fieldsets(fieldsets)
         else:
             readonly_fields = self.get_readonly_fields(request, obj)
-        adminForm = helpers.AdminForm(
+        admin_form = helpers.AdminForm(
             form,
             list(fieldsets),
             # Clear prepopulated fields on a view-only form to avoid a crash.
@@ -1840,13 +1835,13 @@ class ModelAdmin(BaseModelAdmin):
             readonly_fields,
             model_admin=self,
         )
-        media = self.media + adminForm.media
+        media = self.media + admin_form.media
 
         inline_formsets = self.get_inline_formsets(
             request, formsets, inline_instances, obj
         )
         for inline_formset in inline_formsets:
-            media = media + inline_formset.media
+            media += inline_formset.media
 
         if add:
             title = _("Add %s")
@@ -1856,9 +1851,9 @@ class ModelAdmin(BaseModelAdmin):
             title = _("View %s")
         context = {
             **self.admin_site.each_context(request),
-            "title": title % opts.verbose_name,
+            "title": title % self.opts.verbose_name,
             "subtitle": str(obj) if obj else None,
-            "adminform": adminForm,
+            "adminform": admin_form,
             "object_id": object_id,
             "original": obj,
             "is_popup": IS_POPUP_VAR in request.POST or IS_POPUP_VAR in request.GET,
@@ -1896,7 +1891,7 @@ class ModelAdmin(BaseModelAdmin):
     def _get_edited_object_pks(self, request, prefix):
         """Return POST data values of list_editable primary keys."""
         pk_pattern = re.compile(
-            r"{}-\d+-{}$".format(re.escape(prefix), self.model._meta.pk.name)
+            r"{}-\d+-{}$".format(re.escape(prefix), self.opts.pk.name)
         )
         return [value for key, value in request.POST.items() if pk_pattern.match(key)]
 
@@ -1923,8 +1918,7 @@ class ModelAdmin(BaseModelAdmin):
         """
         from django.contrib.admin.views.main import ERROR_FLAG
 
-        opts = self.model._meta
-        app_label = opts.app_label
+        app_label = self.opts.app_label
         if not self.has_view_or_change_permission(request):
             raise PermissionDenied
 
@@ -2018,15 +2012,17 @@ class ModelAdmin(BaseModelAdmin):
             )
             if formset.is_valid():
                 changecount = 0
-                for form in formset.forms:
-                    if form.has_changed():
-                        obj = self.save_form(request, form, change=True)
-                        self.save_model(request, obj, form, change=True)
-                        self.save_related(request, form, formsets=[], change=True)
-                        change_msg = self.construct_change_message(request, form, None)
-                        self.log_change(request, obj, change_msg)
-                        changecount += 1
-
+                with transaction.atomic(using=router.db_for_write(self.model)):
+                    for form in formset.forms:
+                        if form.has_changed():
+                            obj = self.save_form(request, form, change=True)
+                            self.save_model(request, obj, form, change=True)
+                            self.save_related(request, form, formsets=[], change=True)
+                            change_msg = self.construct_change_message(
+                                request, form, None
+                            )
+                            self.log_change(request, obj, change_msg)
+                            changecount += 1
                 if changecount:
                     msg = ngettext(
                         "%(count)s %(name)s was changed successfully.",
@@ -2034,7 +2030,7 @@ class ModelAdmin(BaseModelAdmin):
                         changecount,
                     ) % {
                         "count": changecount,
-                        "name": model_ngettext(opts, changecount),
+                        "name": model_ngettext(self.opts, changecount),
                     }
                     self.message_user(request, msg, messages.SUCCESS)
 
@@ -2065,7 +2061,7 @@ class ModelAdmin(BaseModelAdmin):
 
         context = {
             **self.admin_site.each_context(request),
-            "module_name": str(opts.verbose_name_plural),
+            "module_name": str(self.opts.verbose_name_plural),
             "selection_note": _("0 of %(cnt)s selected") % {"cnt": len(cl.result_list)},
             "selection_note_all": selection_note_all % {"total_count": cl.result_count},
             "title": cl.title,
@@ -2090,7 +2086,7 @@ class ModelAdmin(BaseModelAdmin):
             request,
             self.change_list_template
             or [
-                "admin/%s/%s/change_list.html" % (app_label, opts.model_name),
+                "admin/%s/%s/change_list.html" % (app_label, self.opts.model_name),
                 "admin/%s/change_list.html" % app_label,
                 "admin/change_list.html",
             ],
@@ -2111,8 +2107,7 @@ class ModelAdmin(BaseModelAdmin):
 
     def _delete_view(self, request, object_id, extra_context):
         "The 'delete' admin view for this model."
-        opts = self.model._meta
-        app_label = opts.app_label
+        app_label = self.opts.app_label
 
         to_field = request.POST.get(TO_FIELD_VAR, request.GET.get(TO_FIELD_VAR))
         if to_field and not self.to_field_allowed(request, to_field):
@@ -2126,7 +2121,7 @@ class ModelAdmin(BaseModelAdmin):
             raise PermissionDenied
 
         if obj is None:
-            return self._get_obj_does_not_exist_redirect(request, opts, object_id)
+            return self._get_obj_does_not_exist_redirect(request, self.opts, object_id)
 
         # Populate deleted_objects, a data structure of all related objects that
         # will also be deleted.
@@ -2141,14 +2136,14 @@ class ModelAdmin(BaseModelAdmin):
             if perms_needed:
                 raise PermissionDenied
             obj_display = str(obj)
-            attr = str(to_field) if to_field else opts.pk.attname
+            attr = str(to_field) if to_field else self.opts.pk.attname
             obj_id = obj.serializable_value(attr)
             self.log_deletion(request, obj, obj_display)
             self.delete_model(request, obj)
 
             return self.response_delete(request, obj_display, obj_id)
 
-        object_name = str(opts.verbose_name)
+        object_name = str(self.opts.verbose_name)
 
         if perms_needed or protected:
             title = _("Cannot delete %(name)s") % {"name": object_name}
@@ -2165,7 +2160,7 @@ class ModelAdmin(BaseModelAdmin):
             "model_count": dict(model_count).items(),
             "perms_lacking": perms_needed,
             "protected": protected,
-            "opts": opts,
+            "opts": self.opts,
             "app_label": app_label,
             "preserved_filters": self.get_preserved_filters(request),
             "is_popup": IS_POPUP_VAR in request.POST or IS_POPUP_VAR in request.GET,
@@ -2192,8 +2187,7 @@ class ModelAdmin(BaseModelAdmin):
             raise PermissionDenied
 
         # Then get the history for this object.
-        opts = model._meta
-        app_label = opts.app_label
+        app_label = self.opts.app_label
         action_list = (
             LogEntry.objects.filter(
                 object_id=unquote(object_id),
@@ -2216,9 +2210,9 @@ class ModelAdmin(BaseModelAdmin):
             "page_range": page_range,
             "page_var": PAGE_VAR,
             "pagination_required": paginator.count > 100,
-            "module_name": str(capfirst(opts.verbose_name_plural)),
+            "module_name": str(capfirst(self.opts.verbose_name_plural)),
             "object": obj,
-            "opts": opts,
+            "opts": self.opts,
             "preserved_filters": self.get_preserved_filters(request),
             **(extra_context or {}),
         }
@@ -2229,7 +2223,7 @@ class ModelAdmin(BaseModelAdmin):
             request,
             self.object_history_template
             or [
-                "admin/%s/%s/object_history.html" % (app_label, opts.model_name),
+                "admin/%s/%s/object_history.html" % (app_label, self.opts.model_name),
                 "admin/%s/object_history.html" % app_label,
                 "admin/object_history.html",
             ],
@@ -2268,7 +2262,7 @@ class ModelAdmin(BaseModelAdmin):
             formset_params = self.get_formset_kwargs(request, obj, inline, prefix)
             formset = FormSet(**formset_params)
 
-            def user_deleted_form(request, obj, formset, index):
+            def user_deleted_form(request, obj, formset, index, inline):
                 """Return whether or not the user deleted the form."""
                 return (
                     inline.has_delete_permission(request, obj)
@@ -2279,7 +2273,7 @@ class ModelAdmin(BaseModelAdmin):
             # data won't be in request.POST), unless the form was deleted.
             if not inline.has_change_permission(request, obj if change else None):
                 for index, form in enumerate(formset.initial_forms):
-                    if user_deleted_form(request, obj, formset, index):
+                    if user_deleted_form(request, obj, formset, index, inline):
                         continue
                     form._errors = {}
                     form.cleaned_data = form.initial
@@ -2319,11 +2313,11 @@ class InlineModelAdmin(BaseModelAdmin):
         super().__init__()
         if self.verbose_name_plural is None:
             if self.verbose_name is None:
-                self.verbose_name_plural = self.model._meta.verbose_name_plural
+                self.verbose_name_plural = self.opts.verbose_name_plural
             else:
                 self.verbose_name_plural = format_lazy("{}s", self.verbose_name)
         if self.verbose_name is None:
-            self.verbose_name = self.model._meta.verbose_name
+            self.verbose_name = self.opts.verbose_name
 
     @property
     def media(self):
